from enum import Enum


class LanguageCode(str, Enum):
    ar_xa = 'ar-xa'
    cs_cz = 'cs-cz'
    cy_cy = 'cy-cy'
    cy_gb = 'cy-gb'
    da_dk = 'da-dk'
    de_de = 'de-de'
    el_gr = 'el-gr'
    en_au = 'en-au'
    en_gb = 'en-gb'
    en_in = 'en-in'
    en_us = 'en-us'
    es_es = 'es-es'
    es_mx = 'es-mx'
    es_us = 'es-us'
    fi_fi = 'fi-fi'
    fil_ph = 'fil-ph'
    fr_ca = 'fr-ca'
    fr_fr = 'fr-fr'
    hi_in = 'hi-in'
    hu_hu = 'hu-hu'
    id_id = 'id-id'
    is_is = 'is-is'
    it_it = 'it-it'
    ja_jp = 'ja-jp'
    ko_kr = 'ko-kr'
    nb_no = 'nb-no'
    nl_nl = 'nl-nl'
    pl_pl = 'pl-pl'
    pt_br = 'pt-br'
    pt_pt = 'pt-pt'
    ro_ro = 'ro-ro'
    ru_ru = 'ru-ru'
    sv_se = 'sv-se'
    th_th = 'th-th'
    tr_tr = 'tr-tr'
    vi_vn = 'vi-vn'
    yue_cn = 'yue-cn'
    zh_cn = 'zh-cn'
    zh_tw = 'zh-tw'


class Psd2LanguageCode(str, Enum):
    en_gb = 'en-gb'
    bg_bg = 'bg-bg'
    cs_cz = 'cs-cz'
    da_dk = 'da-dk'
    de_de = 'de-de'
    ee_et = 'ee-et'
    el_gr = 'el-gr'
    es_es = 'es-es'
    fi_fi = 'fi-fi'
    fr_fr = 'fr-fr'
    ga_ie = 'ga-ie'
    hu_hu = 'hu-hu'
    it_it = 'it-it'
    lv_lv = 'lv-lv'
    lt_lt = 'lt-lt'
    mt_mt = 'mt-mt'
    nl_nl = 'nl-nl'
    pl_pl = 'pl-pl'
    sk_sk = 'sk-sk'
    sl_si = 'sl-si'
    sv_se = 'sv-se'



from typing import List, Optional, Union

from pydantic import Field, validate_call
from vonage_http_client.http_client import HttpClient

from .errors import VerifyError
from .requests import BaseVerifyRequest, Psd2Request, VerifyRequest
from .responses import (
    CheckCodeResponse,
    NetworkUnblockStatus,
    StartVerificationResponse,
    VerifyControlStatus,
    VerifyStatus,
)


class Verify:
    """Calls Vonage's Verify API.

    This class provides methods to interact with Vonage's Verify API for starting verification
    processes.
    """

    def __init__(self, http_client: HttpClient) -> None:
        self._http_client = http_client
        self._sent_data_type = 'form'
        self._auth_type = 'body'

    @property
    def http_client(self) -> HttpClient:
        """The HTTP client used to make requests to the Verify API.

        Returns:
            HttpClient: The HTTP client used to make requests to the Verify API.
        """
        return self._http_client

    @validate_call
    def start_verification(
        self, verify_request: VerifyRequest
    ) -> StartVerificationResponse:
        """Start a verification process.

        Args:
            verify_request (VerifyRequest): The verification request object.

        Returns:
            StartVerificationResponse: The response object containing the verification result.
        """
        return self._make_verify_request(verify_request)

    @validate_call
    def start_psd2_verification(
        self, verify_request: Psd2Request
    ) -> StartVerificationResponse:
        """Start a PSD2 verification process.

        Args:
            verify_request (Psd2Request): The PSD2 verification request object.

        Returns:
            StartVerificationResponse: The response object containing the verification result.
        """
        return self._make_verify_request(verify_request)

    @validate_call
    def check_code(self, request_id: str, code: str) -> CheckCodeResponse:
        """Check a verification code.

        Args:
            request_id (str): The request ID.
            code (str): The verification code.

        Returns:
            CheckCodeResponse: The response object containing the verification result.
        """
        response = self._http_client.post(
            self._http_client.api_host,
            '/verify/check/json',
            {'request_id': request_id, 'code': code},
            self._auth_type,
            self._sent_data_type,
        )
        self._check_for_error(response)
        return CheckCodeResponse(**response)

    @validate_call
    def search(
        self, request: Union[str, List[str]]
    ) -> Union[VerifyStatus, List[VerifyStatus]]:
        """Search for past or current verification requests.

        Args:
            request (str | list[str]): The request ID, or a list of request IDs.

        Returns:
            Union[VerifyStatus, List[VerifyStatus]]: Either the response object
                containing the verification result, or a list of response objects.
        """
        params = {}
        if type(request) == str:
            params['request_id'] = request
        elif type(request) == list:
            params['request_ids'] = request

        response = self._http_client.get(
            self._http_client.api_host, '/verify/search/json', params, self._auth_type
        )

        if 'verification_requests' in response:
            parsed_response = []
            for verification_request in response['verification_requests']:
                parsed_response.append(VerifyStatus(**verification_request))
            return parsed_response
        elif 'error_text' in response:
            error_message = f'Error with the following details: {response}'
            raise VerifyError(error_message)
        else:
            parsed_response = VerifyStatus(**response)
            return parsed_response

    @validate_call
    def cancel_verification(self, request_id: str) -> VerifyControlStatus:
        """Cancel a verification request.

        Args:
            request_id (str): The request ID.

        Returns:
            VerifyControlStatus: The response object containing details of the submitted
                verification control.
        """
        response = self._http_client.post(
            self._http_client.api_host,
            '/verify/control/json',
            {'request_id': request_id, 'cmd': 'cancel'},
            self._auth_type,
            self._sent_data_type,
        )
        self._check_for_error(response)

        return VerifyControlStatus(**response)

    @validate_call
    def trigger_next_event(self, request_id: str) -> VerifyControlStatus:
        """Trigger the next event in the verification process.

        Args:
            request_id (str): The request ID.

        Returns:
            VerifyControlStatus: The response object containing details of the submitted
                verification control.
        """
        response = self._http_client.post(
            self._http_client.api_host,
            '/verify/control/json',
            {'request_id': request_id, 'cmd': 'trigger_next_event'},
            self._auth_type,
            self._sent_data_type,
        )
        self._check_for_error(response)

        return VerifyControlStatus(**response)

    @validate_call
    def request_network_unblock(
        self, network: str, unblock_duration: Optional[int] = Field(None, ge=0, le=86400)
    ) -> NetworkUnblockStatus:
        """Request to unblock a network that has been blocked due to potential fraud detection.

        Note: The network unblock feature is switched off by default.
            Please contact Sales to enable the Network Unblock API for your account.

        Args:
            network (str): The network code of the network to unblock.
            unblock_duration (int, optional): How long (in seconds) to unblock the network for.
        """
        response = self._http_client.post(
            self._http_client.api_host,
            '/verify/network-unblock',
            {'network': network, 'duration': unblock_duration},
            self._auth_type,
        )

        return NetworkUnblockStatus(**response)

    def _make_verify_request(
        self, verify_request: BaseVerifyRequest
    ) -> StartVerificationResponse:
        """Make a verify request.

        This method makes a verify request to the Vonage Verify API.

        Args:
            verify_request (BaseVerifyRequest): The verify request object.

        Returns:
            VerifyResponse: The response object containing the verification result.
        """
        if type(verify_request) == VerifyRequest:
            request_path = '/verify/json'
        elif type(verify_request) == Psd2Request:
            request_path = '/verify/psd2/json'

        response = self._http_client.post(
            self._http_client.api_host,
            request_path,
            verify_request.model_dump(by_alias=True, exclude_none=True),
            self._auth_type,
            self._sent_data_type,
        )
        self._check_for_error(response)

        return StartVerificationResponse(**response)

    def _check_for_error(self, response: dict) -> None:
        """Check for error in the response.

        This method checks if the response contains a non-zero status code
            and raises a VerifyError if this is found.

        Args:
            response (dict): The response object.

        Raises:
            VerifyError: If an error is found in the response.
        """
        if int(response['status']) != 0:
            error_message = f'Error with the following details: {response}'
            raise VerifyError(error_message)



from typing import List, Optional

from pydantic import BaseModel


class StartVerificationResponse(BaseModel):
    request_id: str
    status: str


class CheckCodeResponse(BaseModel):
    request_id: str
    status: str
    event_id: str
    price: str
    currency: str
    estimated_price_messages_sent: Optional[str] = None


class Check(BaseModel):
    date_received: Optional[str] = None
    code: Optional[str] = None
    status: Optional[str] = None
    ip_address: Optional[str] = None


class Event(BaseModel):
    type: Optional[str] = None
    id: Optional[str] = None


class VerifyStatus(BaseModel):
    request_id: Optional[str] = None
    account_id: Optional[str] = None
    status: Optional[str] = None
    number: Optional[str] = None
    price: Optional[str] = None
    currency: Optional[str] = None
    sender_id: Optional[str] = None
    date_submitted: Optional[str] = None
    date_finalized: Optional[str] = None
    first_event_date: Optional[str] = None
    last_event_date: Optional[str] = None
    checks: Optional[List[Check]] = None
    events: Optional[List[Event]] = None
    estimated_price_messages_sent: Optional[str] = None


class VerifyControlStatus(BaseModel):
    status: str
    command: str


class NetworkUnblockStatus(BaseModel):
    network: str
    unblocked_until: str



from logging import getLogger
from typing import Literal, Optional

from pydantic import BaseModel, Field, model_validator
from vonage_utils.types import PhoneNumber

from .language_codes import LanguageCode, Psd2LanguageCode

logger = getLogger('vonage_verify')


class BaseVerifyRequest(BaseModel):
    """Base request object containing the data and options for a verification request."""

    number: PhoneNumber
    country: Optional[str] = Field(None, max_length=2)
    code_length: Optional[Literal[4, 6]] = None
    pin_expiry: Optional[int] = Field(None, ge=60, le=3600)
    next_event_wait: Optional[int] = Field(None, ge=60, le=900)
    workflow_id: Optional[int] = Field(None, ge=1, le=7)

    @model_validator(mode='after')
    def check_expiry_and_next_event_timing(self):
        if self.pin_expiry is None or self.next_event_wait is None:
            return self
        if self.pin_expiry % self.next_event_wait != 0:
            logger.warning(
                f'The pin_expiry should be a multiple of next_event_wait.'
                f'\nThe current values are: pin_expiry={self.pin_expiry}, next_event_wait={self.next_event_wait}.'
                f'\nThe value of pin_expiry will be set to next_event_wait.'
            )
            self.pin_expiry = self.next_event_wait
        return self


class VerifyRequest(BaseVerifyRequest):
    """Request object for a verification request.

    You must set the `number` and `brand` fields.
    """

    brand: str = Field(..., max_length=18)
    sender_id: Optional[str] = Field(None, max_length=11)
    lg: Optional[LanguageCode] = None
    pin_code: Optional[str] = Field(None, min_length=4, max_length=10)


class Psd2Request(BaseVerifyRequest):
    """Request object for a PSD2 verification request.

    You must set the `number`, `payee` and `amount` fields.
    """

    payee: str = Field(..., max_length=18)
    amount: float
    lg: Optional[Psd2LanguageCode] = None



from .errors import VerifyError
from .language_codes import LanguageCode, Psd2LanguageCode
from .requests import Psd2Request, VerifyRequest
from .responses import (
    CheckCodeResponse,
    NetworkUnblockStatus,
    StartVerificationResponse,
    VerifyControlStatus,
    VerifyStatus,
)
from .verify import Verify

__all__ = [
    'Verify',
    'VerifyError',
    'LanguageCode',
    'Psd2LanguageCode',
    'Psd2Request',
    'VerifyRequest',
    'CheckCodeResponse',
    'NetworkUnblockStatus',
    'StartVerificationResponse',
    'VerifyControlStatus',
    'VerifyStatus',
]
